﻿using Demo;
using Ice;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Client
{
    public partial class ClientForm : Form
    {
        private int _delay;
        private List<MockClient> _lstMockClient = new List<MockClient>(50);

        public ClientForm()
        {
            InitializeComponent();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            try
            {
                int mockClientNum = Int32.Parse(tbMockClientNum.Text);
                _delay = Int32.Parse(tbDelay.Text);

                btnStart.Enabled = false;
                for (int i = 0; i < mockClientNum; i++)
                {
                    MockClient mockClient = _lstMockClient[i];
                    mockClient.AutoEvent.Set();
                                            
                }
                btnStart.Enabled = true;
            }
            catch (System.Exception ex)
            {
                Log(ex.Message);
            }
        }

        private void ClientForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            foreach (var mockClient in _lstMockClient)
            {
                mockClient.Thread.Abort();
                mockClient.Communicator.destroy();
            }
        }

        private void ClientForm_Load(object sender, EventArgs e)
        {
            Initialize();
        }

        private void Initialize()
        {
            for (int i = 0; i < 50; i++)
            {
                Communicator communicator = Util.initialize();
                HelloPrx hello = HelloPrxHelper.checkedCast(communicator.stringToProxy("hello:tcp -p 10000"));
                AutoResetEvent autoEvent = new AutoResetEvent(false);
                Thread t = new Thread(() =>
                {
                    while (true)
                    {
                        autoEvent.WaitOne();
                        BeginInvoke(new Action<string>(Log), "ManagedThreadId: " + Thread.CurrentThread.ManagedThreadId.ToString());
                        hello.sayHello(_delay);
                    }
                });
                t.Start();
                MockClient mockClient = new MockClient(communicator, hello, t, autoEvent);
                _lstMockClient.Add(mockClient);
            }
        }

        private void Log(string msg)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("{0} {1}", DateTime.Now.ToString("HH:mm:ss:fff"), msg);
            sb.AppendLine();

            tbLog.Text += sb.ToString();

            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();
        }
    }

    public class MockClient
    {
        public Communicator Communicator { get; set; }
        public HelloPrx HelloPrx { get; set; }
        public Thread Thread { get; set; }
        public AutoResetEvent AutoEvent { get; set; }

        public MockClient(Communicator comm, HelloPrx prx, Thread t, AutoResetEvent autoEvent)
        {
            Communicator = comm;
            HelloPrx = prx;
            Thread = t;
            AutoEvent = autoEvent;
        }
    }
}
